% categorization_no_abstraction_FIG2
% This script makes Figure 2 in the accompanying book chapter
% What do neurons really want? The role of semantics in coritcal representations
% Gabriel Kreiman

% A couple of things need to be set up first for this script to work:
% [1] This script assumes that there is a folder called 'cropped227' 
% [2] This folder should have multiple subfolders, each subfolder contains images 
% [3] The subfolders are based on imagenet categories
% Some of the code is hard wired to specifically examine the example in Figure 2, with the specific categories indicated there
% However, it should be pretty straightforward to tweek the code as indicated below to use different images and categories
% The images are NOT provided. They can be downloaded from:
% http://image-net.org/
% (free account required)
% [4] The images are assumed to be 227 pixels x 227 pixels x 3 colors
% [5] The extension of each image file is given by image_file_extension (by default: '.JPEG')

%% Parameters
codename='categorization_no_abstraction_FIG2';

image_file_extension='.JPEG';

classification_layer=23;            % layer used for dreaming in categorization_no_abstraction_deepdream
                                    % layer used to compute features

image_size=227;

toplot=1;                           % Set to 1 to make plots
tosave=1;                           % Set to 1 to save plots

d=dir('cropped227');
if (length(d)<4)
    fprintf(1,'error! I cannot find the cropped227 directory or the directory does not contain at least two subfolders\n');
    return;
end

%% Create image datastore
if (~exist('imds','var'))
    fprintf(1,'Opening datastore...\n');
    imds=imageDatastore('cropped227','FileExtensions','.JPEG','IncludeSubfolders',1,'LabelSource','foldernames');
end
a=imds.countEachLabel;
n_per_category=a.Count;

%% Open AlexNet
if (~exist('net','var'))
    fprintf(1,'Opening alexnet...\n');
    net = alexnet;
end

%% Compute activations
if (~exist('features','var'))
    fprintf(1,'Calculating activations...\n');
    features=activations(net,imds,classification_layer,'OutputAs','rows');
end

% n_per_category'
% 1846         972        1366        1165
% labrador, fireant, desk, sportscar

% Here the code can be changed to examine other units
labrador_channel=209;   % Labrador channel = 209
desk_channel=527;       % Desk channel = 527

if (toplot)
    fprintf(1,'Making figures ...\n');
    a=cumsum(n_per_category);
    for i=1:2
        if (i==1)
            channel=labrador_channel;
        else
            channel=desk_channel;
        end
        
        hs=subplot(2,2,i);
        hp=plot(features(:,channel),'ko');
        set(hs,'TickDir','Out');
        set(hs,'Box','Off');
        xlabel('Image number');
        ylabel('Activation (fc8, ch209)');
        hold on;
        if (i==1)
            i1=1;
            i2=a(1);
            % Load deep dream results (output from categorization_no_abstraction_deepdream)
            load deep_dream_channel209_labradorretriever_niterations100_levels4_layer23.mat
        else
            i1=a(2)+1;
            i2=a(3);
            % Load deep dream results (output from categorization_no_abstraction_deepdream)
            load deep_dream_channel527_desk_niterations200_levels5_layer23.mat
        end
        hp=plot(i1:i2,features(i1:i2,channel),'ro');
        
        img=imresize(I,[image_size image_size]);
        img2=round(255*img);
        features_dream=activations(net,img2,classification_layer,'OutputAs','rows');
        
        I=round(255*I);
        features_dream2=activations(net,I,classification_layer,'OutputAs','channels');
        features_dream2=features_dream2(:,:,channel);
        features_dream2=max(max(features_dream2));
        
        if (i==1)
            hp=plot(a(1),features_dream(channel),'bs');
            hp=plot(a(1),features_dream2,'g^');
        else
            hp=plot(a(3),features_dream(channel),'bs');
            hp=plot(a(3),features_dream2,'g^');
        end
        
        curr_axis=axis;
        for j=1:4
            hp=plot([a(j) a(j)],[curr_axis(3) curr_axis(4)],'k:');
            set(hp,'Color',[0.5 0.5 0.5]);
            set(hp,'LineWidth',1);
        end
        
        axis([0 max(a)+20 curr_axis(3) curr_axis(4)+5]);
    end
    
    if (tosave)
        print_s=sprintf('print -depsc %s.eps',codename);
        disp(print_s);
        eval(print_s);
    end
end
    
    
