% categorization_no_abstraction_deepdream

% Deep Dream Images Using AlexNet
% Based on
% https://www.mathworks.com/help/deeplearning/examples/deep-dream-images-using-alexnet.html

%% Parameters
layer = 23;                 % last layer before classification
                            % this can be modified to create efficient images for other layers
labrador_channel = 209;     % the code is set to create Figure 2 in the chapter. It can be easily changed to generate images for other channels here
desk_channel = 527;     

iterations = 200;           % Number of iterations for deepdream algorithm
levels = 5;                 % Number of levels for deepdream algorithm

tosave=1;

%%Load Pretrained AlexNet Network
if (~exist('net','var'))
    fprintf(1,'Loading alexnet...\n');
    net = alexnet;
end

for i=1:2
    if (i==1)
        channels=labrador_channel;
    else
        channels=desk_channel;
    end
    
    txt=net.Layers(end).ClassNames(channels);
    fprintf(1,'Processing i=%d channels=%d %s\n',i,channels,txt{1});
    % The classes are stored in the Classes property of the output layer (the last layer). You can view the names of the selected classes by selecting the entries in channels.
    %
    % net.Layers(end).ClassNames(channels)
    % ans = 61 categorical array
    %      hen
    %      Yorkshire terrier
    %      Shetland sheepdog
    %      fountain
    %      theater curtain
    %      geyser
    %

    fprintf(1,'Dreaming ...\n');
    I = deepDreamImage(net,layer,channels,'Verbose',false,'NumIterations',iterations,'PyramidLevels',levels);

    figure
    imshow(I)

    if (tosave)
        core_filename=sprintf('deep_dream_channel%d_desk_niterations%d_levels%d_layer%d',channels,iterations,levels,layer);
        save_s=sprintf('save %s.mat I',core_filename);
        disp(save_s);eval(save_s);

        %print_s=sprintf('print -djpeg %s.jpg',core_filename);
        %disp(print_s);eval(print_s);
        print_s=sprintf('print -depsc %s.eps',core_filename);
        disp(print_s);eval(print_s);
    end
    if (i==1)
        close;
    end
end

